@@ -30,27 +30,27 @@ using System.Xml.Serialization;
 using Beagle.Util;
 
 namespace Beagle {
+
+	public enum PropertyType {
+		Text     = 1,
+		Keyword  = 2,
+		Date     = 3
+	}
 	
-	[XmlInclude (typeof (Property))]
-		public class Property : IComparable, ICloneable {
-
-		bool   isKeyword;
-		bool   isSearched;
-		string key;
-		string value;
+	public class Property : IComparable, ICloneable {
+		
+		PropertyType type;
+		string       key;
+		string       value;
+		bool         is_searched;
+		bool         is_mutable;
 
 		[XmlAttribute]
-		public bool IsKeyword {
-			get { return isKeyword; }
-			set { isKeyword = value; }
+		public PropertyType Type {
+			get { return type; }
+			set { type = value; }
 		}
-
-		[XmlAttribute]
-		public bool IsSearched {
-			get { return isSearched; }
-			set { isSearched = value; }
-		}
-
+		
 		[XmlAttribute]
 		public string Key {
 			get { return key; }
@@ -62,7 +62,25 @@ namespace Beagle {
 			get { return value; }
 			set { this.value = StringFu.CleanupInvalidXmlCharacters (value); }
 		}
-		
+
+		// If IsSearched is true, this property will can be matched by a
+		// general match-any-propety query.
+		// You can always query against the specific property, even if
+		// IsSearched is false.
+		[XmlAttribute]
+		public bool IsSearched {
+			get { return is_searched; }
+			set { is_searched = value; }
+		}
+
+		// When IsMutable is true, the property is stored in the secondary
+		// index so that it can more efficiently be changed later on.
+		[XmlAttribute]
+		public bool IsMutable {
+			get { return is_mutable; }
+			set { is_mutable = value; }
+		}
+
 		/////////////////////////////////////
 
 		public Property () { }
@@ -81,45 +99,61 @@ namespace Beagle {
 			if (other_property == null)
 				return this.GetHashCode ().CompareTo (other.GetHashCode ());
 
-			return String.Compare (this.Key, other_property.Key);
+			int rv;
+			rv = String.Compare (this.Key, other_property.Key);
+			if (rv != 0)
+				return rv;
+
+			return String.Compare (this.Value, other_property.Value);
 		}
 
 		public object Clone ()
 		{
-			object clone = this.MemberwiseClone ();
-
-			return clone;
+			return this.MemberwiseClone ();
 		}
-		
-		static public Property New (string key, object value)
+
+		static public Property New (string key, string value)
 		{
+			if (value == null)
+				return null;
+
 			Property p = new Property ();
-			p.isKeyword = false;
-			p.IsSearched = true;
-			p.key = key;
-			p.Value = value != null ? value.ToString () : null;
+			p.type = PropertyType.Text;
+			p.Key = key;
+			p.Value = value;
+			p.is_searched = true;
 			return p;
 		}
 
 		static public Property NewKeyword (string key, object value)
 		{
-			Property p = Property.New (key, value);
-			p.isKeyword = true;
-			p.isSearched = true;
+			if (value == null)
+				return null;
+
+			Property p = new Property ();
+			p.type = PropertyType.Keyword;
+			p.Key = key;
+			p.Value = value.ToString ();
+			p.is_searched = true;
 			return p;
 		}
 
 		static public Property NewUnsearched (string key, object value)
-		{
-			Property p = Property.New (key, value);
-			p.isKeyword = true;
-			p.isSearched = false;
+		{		
+			if (value == null)
+				return null;
+
+			Property p = new Property ();
+			p.type = PropertyType.Keyword;
+			p.Key = key;
+			p.Value = value.ToString ();
+			p.is_searched = false;
 			return p;
 		}
 
 		static public Property NewBool (string key, bool value)
 		{
-			return NewUnsearched (key, value ? "true" : "false");
+			return Property.NewUnsearched (key, value ? "true" : "false");
 		}
 
 		static public Property NewFlag (string key)
@@ -129,7 +163,26 @@ namespace Beagle {
 
 		static public Property NewDate (string key, DateTime dt)
 		{
-			return NewUnsearched (key, StringFu.DateTimeToString (dt));
+			Property p = new Property ();
+			p.type = PropertyType.Date;
+			p.Key = key;
+			p.Value = StringFu.DateTimeToString (dt);
+			p.is_searched = true;
+			return p;
+		}
+
+		static public Property NewDateFromString (string key, string value)
+		{
+			if (value == null)
+				return null;
+
+			Property p = new Property ();
+			p.type = PropertyType.Date;
+			p.Key = key;
+			// FIXME: Should probably check that value is a valid date string.
+			p.Value = value;
+			p.is_searched = true;
+			return p;
 		}
 
 		override public string ToString ()
