@@ -27,7 +27,7 @@
 using System;
 using System.Diagnostics;
 using Gtk;
-using BU = Beagle.Util;
+using Beagle.Util;
 
 namespace Beagle.Tile {
 
@@ -169,12 +169,11 @@ namespace Beagle.Tile {
 			command = "desktop-launch";
 			expects_uris = true;
 #else		       
-			BU.GnomeVFSMimeApplication app;
-			app = BU.GnomeIconLookup.GetDefaultAction (hit.MimeType);
-			
+			GnomeFu.VFSMimeApplication app;
+			app = GnomeFu.GetDefaultAction (hit.MimeType);
 			if (app.command != null) {
 				command = app.command;
-				expects_uris = (app.expects_uris != BU.GnomeVFSMimeApplicationArgumentType.Path);
+				expects_uris = (app.expects_uris != GnomeFu.VFSMimeApplicationArgumentType.Path);
 			}
 #endif			
 			if (command == null) {
