@@ -29,12 +29,16 @@ using System.Diagnostics;
 using System.IO;
 using System.Text;
 
+using Mono.Unix;
+
 using BU = Beagle.Util;
 
 namespace Beagle.Tile {
 	
 	[HitFlavor (Name="Docs", Rank=400, Emblem="", Color="#f5f5fe",
 		    Type="File", MimeType="application/vnd.sun.xml.writer")]
+	[HitFlavor (Name="Docs", Rank=400, Emblem="", Color="#f5f5fe",
+		    Type="File", MimeType = "application/vnd.sun.xml.writer.template")]
 	[HitFlavor (Name="Docs", Rank=400, Emblem="", Color="#f5f5fe",
 		    Type="File", MimeType="application/msword")]
 	[HitFlavor (Name="Docs", Rank=400, Emblem="", Color="#f5f5fe",
@@ -47,6 +51,14 @@ namespace Beagle.Tile {
 		    Type="File", MimeType="application/x-abiword")]
 	[HitFlavor (Name="Docs", Rank=400, Emblem="", Color="#f5f5fe",
 		    Type="File", MimeType="application/rtf")]
+	[HitFlavor (Name="Docs", Rank=400, Emblem="", Color="#f5f5fe",
+		    Type="File", MimeType="application/x-chm")]
+		
+	// OO 2.0 (writer) formats
+	[HitFlavor (Name="Docs", Rank=400, Emblem="", Color="#f5f5fe",
+		    Type="File", MimeType="application/vnd.oasis.opendocument.text")]
+	[HitFlavor (Name="Docs", Rank=400, Emblem="", Color="#f5f5fe",
+		    Type="File", MimeType = "application/vnd.oasis.opendocument.text.template")]
 		
 		public class TileDocs : TileFile {
 			public TileDocs (Hit _hit) : base (_hit, "template-docs.html")
@@ -62,11 +74,11 @@ namespace Beagle.Tile {
 					Template ["Title"] = Hit ["dc:title"];
 
                                 if (Hit ["fixme:page-count"] != null)
-					strPagesAndWords.Append("Pages: "+Hit ["fixme:page-count"]);
+					strPagesAndWords.Append (Catalog.GetString ("Pages:") + " " + Hit ["fixme:page-count"]);
 				//Template ["PagesAndWords"] = Hit ["fixme:page-count"];
 
                                 if (Hit ["fixme:word-count"] != null)
-					strPagesAndWords.Append (" - Words: "+Hit ["fixme:word-count"]);
+					strPagesAndWords.Append (" - " + Catalog.GetString ("Words:") + " " + Hit ["fixme:word-count"]);
 				//Template ["Words"] = Hit ["fixme:word-count"];
 				Template["PagesAndWords"] = strPagesAndWords.ToString();
 			}
