@@ -39,7 +39,7 @@ namespace Beagle.Tile {
 	 HitFlavor (Name="Music", Rank=400, Emblem="emblem-music.png", Color="#f5f5fe",
 		    Type="File", MimeType="application/ogg"),
 	 HitFlavor (Name="Music", Rank=400, Emblem="emblem-music.png", Color="#f5f5fe",
-		    Type="File", MimeType="application/x-flac"),
+		    Type="File", MimeType="audio/x-flac"),
 	 HitFlavor (Name="Music", Rank=400, Emblem="emblem-music.png", Color="#f5f5fe",
 		    Type="File", MimeType="video/x-ms-asf")]
 
@@ -57,7 +57,7 @@ namespace Beagle.Tile {
 			if (Hit ["fixme:title"] != null && Hit ["fixme:title"].Trim().Length > 0)
 				Template ["Title"] = Hit ["fixme:title"];
 
-			Template ["fixme:artist"] = "Unknown Artist";
+			Template ["Artist"] = "Unknown Artist";
 			if (Hit ["fixme:artist"] != null && Hit ["fixme:artist"].Trim().Length > 0)
 				Template ["Artist"] = Hit ["fixme:artist"];
 
