@@ -38,7 +38,7 @@ namespace Beagle.Daemon {
 
 	public class FilterFactory {
 
-		static private bool Debug = true;
+		static private bool Debug = false;
 
 		static FilterFactory ()
 		{
@@ -118,7 +118,7 @@ namespace Beagle.Daemon {
 
 		static public ICollection CreateFiltersFromPath (string path)
 		{
-			string guessed_mime_type = Beagle.Util.VFS.Mime.GetMimeType (path);
+			string guessed_mime_type = GnomeFu.GetMimeType (path);
 			string extension = Path.GetExtension (path);
 			return CreateFilters (UriFu.PathToFileUri (path), extension, guessed_mime_type);
 		}
@@ -141,6 +141,42 @@ namespace Beagle.Daemon {
 
 		/////////////////////////////////////////////////////////////////////////
 
+		static public TextReader FilterFile (string path)
+		{
+			return FilterFile (path, null);
+		}
+		
+		static public TextReader FilterFile (string path, string mime_type)
+		{
+			if (mime_type == null)
+				mime_type = GnomeFu.GetMimeType (path);
+
+			if (mime_type == null)
+				return null;
+
+			ICollection filters = CreateFilters (UriFu.PathToFileUri (path), Path.GetExtension (path), mime_type);
+			TextReader reader;
+			
+			foreach (Filter candidate_filter in filters) {
+				if (Debug)
+					Logger.Log.Debug ("Testing filter: {0}", candidate_filter);
+						
+				// Open the filter, and hook up the TextReader.
+				if (candidate_filter.Open (path)) {
+					reader = candidate_filter.GetTextReader ();
+					
+					if (Debug)
+						Logger.Log.Debug ("Successfully filtered {0} with {1}", path, candidate_filter);
+					
+					return reader;
+				} else if (Debug) {
+					Logger.Log.Debug ("Unsuccessfully filtered {0} with {1}, falling back", path, candidate_filter);
+				}
+			}		
+
+			return null;
+		} 
+
 		static private bool ShouldWeFilterThis (Indexable indexable)
 		{
 			if (indexable.Filtering == IndexableFiltering.Never
@@ -158,17 +194,20 @@ namespace Beagle.Daemon {
 			return false;
 		}
 
-		static public bool FilterIndexable (Indexable indexable, out Filter filter)
+		static public bool FilterIndexable (Indexable indexable, TextCache text_cache, out Filter filter)
 		{
 			filter = null;
 			ICollection filters = null;
 
+			if (indexable.Filtering == IndexableFiltering.AlreadyFiltered)
+				return false;
+
 			if (! ShouldWeFilterThis (indexable)) {
 				indexable.NoContent = true;
 				return false;
 			}
 
-			string path = indexable.ContentUri.LocalPath;
+			string path = null;
 
 			// First, figure out which filter we should use to deal with
 			// the indexable.
@@ -178,9 +217,11 @@ namespace Beagle.Daemon {
 				filters = CreateFiltersFromMimeType (indexable.MimeType);
 
 			if (indexable.IsNonTransient) {
+				path = indexable.ContentUri.LocalPath;
+
 				// Otherwise sniff the mime-type from the file
 				if (indexable.MimeType == null)
-					indexable.MimeType = Beagle.Util.VFS.Mime.GetMimeType (path);
+					indexable.MimeType = GnomeFu.GetMimeType (path);
 
 				if (filters == null || filters.Count == 0) {
 					filters = CreateFiltersFromIndexable (indexable);
@@ -189,9 +230,9 @@ namespace Beagle.Daemon {
 				if (Directory.Exists (path)) {
 					indexable.MimeType = "inode/directory";
 					indexable.NoContent = true;
-					indexable.Timestamp = Directory.GetLastWriteTime (path);
+					indexable.Timestamp = Directory.GetLastWriteTimeUtc (path);
 				} else if (File.Exists (path)) {
-					indexable.Timestamp = File.GetLastWriteTime (path);
+					indexable.Timestamp = File.GetLastWriteTimeUtc (path);
 				} else {
 					Logger.Log.Warn ("No such file: {0}", path);
 					return false;
@@ -215,29 +256,36 @@ namespace Beagle.Daemon {
 					Logger.Log.Debug ("Testing filter: {0}", candidate_filter);
 				
 				// Hook up the snippet writer.
-				if (candidate_filter.SnippetMode) {
+				if (candidate_filter.SnippetMode && text_cache != null) {
 					if (candidate_filter.OriginalIsText && indexable.IsNonTransient) {
-						TextCache.MarkAsSelfCached (indexable.Uri);
+						text_cache.MarkAsSelfCached (indexable.Uri);
 					} else if (indexable.CacheContent) {
-						TextWriter writer = TextCache.GetWriter (indexable.Uri);
+						TextWriter writer = text_cache.GetWriter (indexable.Uri);
 						candidate_filter.AttachSnippetWriter (writer);
 					}
 				}
-				
+
 				if (indexable.Crawled)
 					candidate_filter.EnableCrawlMode ();
 				
-				// Be extra paranoid: never delete the actual
-				// URI we are indexing.
-				if (indexable.DeleteContent && indexable.Uri != indexable.ContentUri)
-					candidate_filter.DeleteContent = indexable.DeleteContent;
-				
 				// Set the filter's URI
 				candidate_filter.Uri = indexable.Uri;
 				
 				// Open the filter, copy the file's properties to the indexable,
 				// and hook up the TextReaders.
-				if (candidate_filter.Open (path)) {
+
+				bool succesful_open = false;
+				TextReader text_reader;
+				Stream binary_stream;
+
+				if (path != null)
+					succesful_open = candidate_filter.Open (path);
+				else if ((text_reader = indexable.GetTextReader ()) != null)
+					succesful_open = candidate_filter.Open (text_reader);
+				else if ((binary_stream = indexable.GetBinaryStream ()) != null)
+					succesful_open = candidate_filter.Open (binary_stream);
+					
+				if (succesful_open) {
 					foreach (Property prop in candidate_filter.Properties)
 						indexable.AddProperty (prop);
 					indexable.SetTextReader (candidate_filter.GetTextReader ());
@@ -250,6 +298,7 @@ namespace Beagle.Daemon {
 					return true;
 				} else if (Debug) {
 					Logger.Log.Debug ("Unsuccessfully filtered {0} with {1}, falling back", path, candidate_filter);
+					candidate_filter.Cleanup ();
 				}
 			}
 
@@ -259,11 +308,16 @@ namespace Beagle.Daemon {
 			return false;
 		}
 
+		static public bool FilterIndexable (Indexable indexable, out Filter filter)
+		{
+			return FilterIndexable (indexable, null, out filter);
+		}
+
 		static public bool FilterIndexable (Indexable indexable)
 		{
 			Filter filter = null;
 
-			return FilterIndexable (indexable, out filter);
+			return FilterIndexable (indexable, null, out filter);
 		}
 
 		/////////////////////////////////////////////////////////////////////////
