@@ -57,28 +57,15 @@ namespace Beagle.Daemon {
 		public QueryDomain Domain {
 			get { return flavor.Domain; }
 		}
-			
+
 		public bool AcceptQuery (Query query)
 		{
 			return query != null
-				&& ! query.IsEmpty
-				&& query.AllowsSource (Name)
+				&& (query.IsIndexListener || ! query.IsEmpty)
 				&& query.AllowsDomain (Domain)
 				&& iqueryable.AcceptQuery (query);
 		}
-
-		public int GetItemCount ()
-		{
-			int n = -1;
-			try {
-				n = iqueryable.GetItemCount ();
-			} catch (Exception ex) {
-				Logger.Log.Warn ("Couldn't get item count for '{0}'", Name);
-				Logger.Log.Warn (ex);
-			}
-			return n;
-		}
-
+				    
 		public void DoQuery (Query query, IQueryResult result, IQueryableChangeData change_data)
 		{
 			try {
@@ -95,8 +82,8 @@ namespace Beagle.Daemon {
 				return null;
 
 			// Sanity-check: make sure this Hit actually came out of this Queryable
-			if (QueryDriver.GetQueryable (hit.SourceObjectName) != this) {
-				string msg = String.Format ("Queryable mismatch in GetSnippet: {0} vs {1}", hit.SourceObjectName, this);
+			if (QueryDriver.GetQueryable (hit.Source) != this) {
+				string msg = String.Format ("Queryable mismatch in GetSnippet: {0} vs {1}", hit.Source, this);
 				throw new Exception (msg);
 			}
 
@@ -109,5 +96,17 @@ namespace Beagle.Daemon {
 			
 			return null;
 		}
+
+		public QueryableStatus GetQueryableStatus ()
+		{
+		        QueryableStatus status = iqueryable.GetQueryableStatus ();
+
+			if (status == null)
+				status = new QueryableStatus ();
+
+			status.Name = Name;
+
+			return status;
+		}
 	}
 }
