@@ -31,6 +31,7 @@ using System.Net.Sockets;
 using System.Reflection;
 using System.Threading;
 using System.Xml.Serialization;
+using Mono.Unix;
 
 using Beagle.Util;
 
@@ -39,9 +40,10 @@ namespace Beagle.Daemon {
 	class ConnectionHandler {
 
 		private static int connection_count = 0;
-		private static XmlSerializer serializer = new XmlSerializer (typeof (ResponseWrapper), ResponseMessage.Types);
+		public static XmlSerializer serializer = null;
 
 		private object client_lock = new object ();
+		private object blocking_read_lock = new object ();
 
 		private UnixClient client;
 		private RequestMessageExecutor executor = null; // Only set in the keepalive case
@@ -59,14 +61,24 @@ namespace Beagle.Daemon {
 				if (this.client == null)
 					return false;
 
-				try { 
-					serializer.Serialize (this.client.GetStream (), new ResponseWrapper (response));
+				try {
+#if ENABLE_XML_DUMP
+					MemoryStream mem_stream = new MemoryStream ();
+					XmlFu.SerializeUtf8 (serializer, mem_stream, new ResponseWrapper (response));
+					mem_stream.Seek (0, SeekOrigin.Begin);
+					StreamReader r = new StreamReader (mem_stream);
+					Logger.Log.Debug ("Sending response:\n{0}", r.ReadToEnd ());
+					mem_stream.Seek (0, SeekOrigin.Begin);
+					mem_stream.WriteTo (this.client.GetStream ());
+					mem_stream.Close ();
+#else
+					XmlFu.SerializeUtf8 (serializer, this.client.GetStream (), new ResponseWrapper (response));
+#endif
 					// Send an end of message marker
 					this.client.GetStream ().WriteByte (0xff);
-					this.client.Flush ();
-				} catch (IOException e) {
-					// The socket was shut down, so we can't write
-					// any more.
+					this.client.GetStream ().Flush ();
+				} catch (Exception e) {
+					Logger.Log.Debug ("Caught an exception sending response; socket shut down: {0}", e.Message);
 					return false;
 				}
 
@@ -80,8 +92,10 @@ namespace Beagle.Daemon {
 			// close a socket and have it exit out of a blocking
 			// read, so we have to abort the thread it's blocking
 			// in.
-			if (this.in_blocking_read) {
-				this.thread.Abort ();
+			lock (this.blocking_read_lock) {
+				if (this.in_blocking_read) {
+					this.thread.Abort ();
+				}
 			}
 		}
 		
@@ -164,18 +178,24 @@ namespace Beagle.Daemon {
 				bytes_read = 0;
 
 				try {
-					this.in_blocking_read = true;
+					lock (this.blocking_read_lock)
+						this.in_blocking_read = true;
+
 					lock (this.client_lock) {
 						// The connection may have been closed within this loop.
 						if (this.client != null)
 							bytes_read = this.client.GetStream ().Read (network_data, 0, 4096);
 					}
-					this.in_blocking_read = false;
-				} catch (IOException) {
+
+					lock (this.blocking_read_lock)
+						this.in_blocking_read = false;
+				} catch (Exception e) {
 					// Aborting the thread mid-read will
 					// cause an IOException to be thorwn,
 					// which sets the ThreadAbortException
 					// as its InnerException.
+					if (!(e is IOException || e is ThreadAbortException))
+						throw;
 
 					Logger.Log.Debug ("Bailing out of HandleConnection -- shutdown requested");
 					Server.MarkHandlerAsKilled (this);
@@ -187,7 +207,7 @@ namespace Beagle.Daemon {
 
 				if (bytes_read > 0) {
 					// 0xff signifies end of message
-					end_index = Array.IndexOf (network_data, (byte) 0xff);
+					end_index = ArrayFu.IndexOfByte (network_data, (byte) 0xff);
 
 					buffer_stream.Write (network_data, 0,
 							     end_index == -1 ? bytes_read : end_index);
@@ -208,7 +228,7 @@ namespace Beagle.Daemon {
 
 			try {
 				RequestWrapper wrapper = (RequestWrapper) req_serializer.Deserialize (buffer_stream);
-
+				
 				req = wrapper.Message;
 			} catch (Exception e) {
 				resp = new ErrorResponse (e);
@@ -237,7 +257,8 @@ namespace Beagle.Daemon {
 					exec.AsyncResponseEvent += OnAsyncResponse;
 				}
 
-				resp = exec.Execute (req);
+				if (exec != null)
+					resp = exec.Execute (req);
 			}
 
 			// It's okay if the response is null; this means
@@ -269,7 +290,7 @@ namespace Beagle.Daemon {
 
 	public class Server {
 
-		private string name;
+		private string socket_path;
 		private UnixListener listener;
 		private static Hashtable live_handlers = new Hashtable ();
 		private bool running = false;
@@ -281,8 +302,9 @@ namespace Beagle.Daemon {
 			// Use the default name when passed null
 			if (name == null)
 				name = "socket";
-			this.name = name;
-			this.listener = new UnixListener (Path.Combine (PathFinder.StorageDir, name));
+
+			this.socket_path = Path.Combine (PathFinder.GetRemoteStorageDir (true), name);
+			this.listener = new UnixListener (this.socket_path);
 		}
 
 		public Server () : this (null)
@@ -319,7 +341,9 @@ namespace Beagle.Daemon {
 			this.listener.Start ();
 			this.running = true;
 
-			Shutdown.WorkerStart (this, String.Format ("server '{0}'", name));
+			Shutdown.WorkerStart (this, String.Format ("server '{0}'", socket_path));
+			if (ConnectionHandler.serializer == null)
+				ConnectionHandler.serializer = new XmlSerializer (typeof (ResponseWrapper), ResponseMessage.Types);
 
 			while (this.running) {
 				UnixClient client;
@@ -354,7 +378,7 @@ namespace Beagle.Daemon {
 			
 			Shutdown.WorkerFinished (this);
 
-			Logger.Log.Debug ("Server '{0}' shut down", this.name);
+			Logger.Log.Debug ("Server '{0}' shut down", this.socket_path);
 		}
 
 		public void Start ()
@@ -366,6 +390,7 @@ namespace Beagle.Daemon {
 		{
 			this.running = false;
 			this.listener.Stop ();
+			File.Delete (this.socket_path);
 		}
 
 		//////////////////////////////////////////////////////////////////////////////
